package de.meinefirma.meinprojekt;

import java.io.*;
import java.util.Arrays;
import org.apache.ftpserver.FtpServer;
import org.apache.ftpserver.ftplet.FtpException;
import org.junit.*;

public class FtpTest
{
   private static final int    FTP_PORT           = 2121;
   private static final String FTP_HOST           = "localhost";
   private static final String FTPHOME_DIR        = "target/FtpHome";
   private static final String FTPUSERSPROPS_FILE = "target/FtpUsers.properties";
   private static final String READ_USER_NAME     = "ReadUserName";
   private static final String READ_USER_PWD      = "ReadUserPwd";
   private static final String WRITE_USER_NAME    = "WriteUserName";
   private static final String WRITE_USER_PWD     = "WriteUserPwd";
   private FtpServer ftpServer;

   @Before
   public void startFtpServer() throws FtpException, IOException
   {
      ftpServer = FtpUtils.createFtpServer( FTP_PORT, FTPHOME_DIR,
            READ_USER_NAME, READ_USER_PWD, WRITE_USER_NAME, WRITE_USER_PWD, FTPUSERSPROPS_FILE );
      ftpServer.start();
   }

   @After
   public void stoppFtpServer() throws InterruptedException
   {
      // Folgende Zeile kann aktiviert werden, um den FTP-Server ueber den Webbrowser anzusprechen, ueber:
      // ftp://WriteUserName:WriteUserPwd@localhost:2121
      // Thread.sleep( 55000 );
      ftpServer.stop();
   }

   @Test
   public void testFtp() throws IOException
   {
      final String testLocalSrcFile = "TestSrc.txt";
      final String testLocalDstFile = "target/TestDst.txt";
      final String testRemoteFile   = "Test.txt";

      File testFile = new File( testLocalSrcFile );
      if( !testFile.exists() ) testFile.createNewFile();

      // Teste Upload, zuerst ohne und dann mit Schreibberechtigung:
      Assert.assertFalse( "READ_USER", FtpUtils.upload( testLocalSrcFile, testRemoteFile, FTP_HOST, FTP_PORT,
            READ_USER_NAME, READ_USER_PWD, false ) );
      Assert.assertTrue( "WRITE_USER", FtpUtils.upload( testLocalSrcFile, testRemoteFile, FTP_HOST, FTP_PORT,
            WRITE_USER_NAME, WRITE_USER_PWD, false ) );
      Assert.assertTrue( "testRemoteFile.exists", new File( FTPHOME_DIR, testRemoteFile ).exists() );

      // Teste Download:
      Assert.assertTrue( "Download", FtpUtils.download( testLocalDstFile, testRemoteFile, FTP_HOST, FTP_PORT,
            READ_USER_NAME, READ_USER_PWD, false ) );
      Assert.assertTrue( "testLocalDstFile.exists", new File( testLocalDstFile ).exists() );

      // Teste Auflistung:
      String[] remoteFilenameList = FtpUtils.list( FTP_HOST, FTP_PORT, READ_USER_NAME, READ_USER_PWD );
      Assert.assertTrue( "remoteFilenameList", Arrays.asList( remoteFilenameList ).contains( testRemoteFile ) );
   }
}
